/**
 *  !!! WARNING !!!
 *
 * 	DO NOT MODIFY THIS FILE!
 *	IF YOU WANT TO USE A HEADER FILE, ADD ANOTHER ONE AND LEAVE THIS ONE ALONE!
 *
 *
 * v1.2 latest revision: 2017-May-26
 */

#ifndef __MAIN_H__
#define __MAIN_H__

#include <windows.h>

#pragma pack(1)					/// Align structures to 1 byte

#ifdef BUILD_DLL
	#define DLL_EXPORT __declspec(dllexport)
#else
	#define DLL_EXPORT __declspec(dllimport)
#endif

// File pointer move methods
//#define FILE_BEGIN			0x00000000	// defined in windows.h
//#define FILE_CURRENT			0x00000001	// defined in windows.h
//#define FILE_END				0x00000002	// defined in windows.h
#define FILE_IGNORE				0x00000100
#define FILE_RESTORE			0x00001000

// File sizing options
#define FILE_READ_FULL			0xFFFFFFFF

// Compression types
#define COMPRESS_ZLIB			10001000		// ZLIB compression algorithm
#define COMPRESS_DEFLATE		COMPRESS_ZLIB	// same as above
#define COMPRESS_LZF			10002000		// LZF Compression algorithm

/** Archive parameters - Required by Unpakke
 *
 *  Unpakke<->Module communication structure
 */
typedef struct dataArchive {
	char 	*inputPath;					/// Packing=Directory path; Unpacking=Archive filepath
	char 	*outputPath;				/// Packing=Archive filepath; Unpacking=Directory path
	int		numEntries;					/// Number of entries
	DWORD64	archiveSize;				/// Size of the archive
	HANDLE	archiveHandle;				/// file HANDLE of the archive
	byte	*buffer;					/// Packing=raw structure; Unpacking=user's decision
} dataArchive;

/** Archive structure row - Required by Unpakke
 *
 * When packing, dataArchive.buffer should contain buffer of such rows
 */
typedef struct archiveRow {
	char		filePath[MAX_PATH+1];	/// Full, relative (eg, starts from input directory) filepath
	DWORD64		fileSizeRaw;			/// Raw size of the file
	DWORD64		fileSizeZip;			/// Compressed/Encrypted size of the file (if any compression/encryption is applied)
	DWORD64		fileOffset;				/// Relative file offset (first entry is 0, second equals the first entry's size, and so on...)
	DWORD		timestamp;				/// Unix timestamp (not implemented yet)
	FILETIME	time;					/// Windows FILETIME time
} archiveRow;

/** Module information structure - Required by Unpakke
 *
 *  Values must be set inside the init() export function
 */
typedef struct moduleInfo {
	char	codename[20];				/// Module name used as INI section name
	char	author[70];					/// Module author
	char	description[200];			/// Module description (if any)
	char	date[15];					/// Module release date in "dd Mon, YYYY" format!
	char	version[4];					/// Module version, not used, but it's good to have this one
	WORD	min_version;				/// Minimum Unpakke supported version in 0xAABB format (AA = Major version; BB Minor version)
	BOOL	packing;					/// Packing capability (TRUE - packing available; FALSE - packing unavailable)
	BOOL	unpacking;					/// Unpacking capability (TRUE - unpacking available; FALSE - unpacking unavailable)
} moduleInfo;

#ifdef __cplusplus
extern "C" {
#endif
	BOOL DLL_EXPORT pack(dataArchive &a);
	BOOL DLL_EXPORT unpack(dataArchive &a);
	moduleInfo DLL_EXPORT init();
#ifdef __cplusplus
}
#endif

/// Unpakke function prototypes
typedef DWORD			(*_upkkAdler32)(byte *dataBuffer, DWORD bufferSize);
typedef byte*			(*_upkkAllocBuffer)(DWORD bufferSize);
typedef BOOL			(*_upkkCloseStorage)(HANDLE containerHandle);
typedef byte*			(*_upkkCompressBuffer)(DWORD compAlgo, int compLevel, byte *inputBuffer, DWORD inputBufferSize, DWORD *outputBufferSize);
typedef DWORD			(*_upkkCountFiles)(char *baseDir, BOOL recursive);
typedef DWORD			(*_upkkCRC32)(byte *dataBuffer, DWORD bufferSize);
typedef	BOOL			(*_upkkCreateDirectory)(char *szDirName);
typedef byte*			(*_upkkDecompressBuffer)(DWORD compAlgo, byte *inputBuffer, DWORD inputBufferSize, DWORD *outputBufferSize);
typedef BOOL			(*_upkkExtractData)(char *filePath, byte *dataBuffer, DWORD dataLength);
typedef BOOL			(*_upkkExtractFile)(char *filePath, DWORD dataSize, DWORD64 archiveOffset, DWORD dwMoveMethod, HANDLE containerHandle);
typedef byte*			(*_upkkGetData)(DWORD dataSize, DWORD64 archiveOffset, DWORD dwMoveMethod, HANDLE containerHandle);
typedef DWORD64			(*_upkkGetOffset)(HANDLE containerHandle);
typedef DWORD			(*_upkkINIGetInt)(char *lpKeyName, DWORD nDefault);
typedef DWORD			(*_upkkINIGetString)(char *lpKeyName, char *lpDefault, char *lpReturnedString, DWORD nSize);
typedef BOOL			(*_upkkInsertData)(byte *data, DWORD dataSize, DWORD64 archiveOffset, DWORD dwMoveMethod, HANDLE containerHandle);
typedef BOOL			(*_upkkInsertFile)(char *filePath, DWORD64 archiveOffset, DWORD dwMoveMethod, HANDLE containerHandle);
typedef HANDLE			(*_upkkOpenStorage)(char *lpFileName, DWORD dwCreationDisposition);
typedef void			(*_upkkRawCloseFile)(HANDLE hFile);
typedef HANDLE			(*_upkkRawCreateFile)(LPCTSTR filePath, DWORD dwCreationDisposition);
typedef DWORD			(*_upkkRawReadFile)(HANDLE hFile, byte *data, DWORD dataSize);
typedef DWORD			(*_upkkRawWriteFile)(HANDLE hFile, byte *data, DWORD dataSize);
typedef byte*			(*_upkkReadFile)(char *filePath, DWORD *dataSize);
typedef BOOL			(*_upkkReleaseBuffer)(byte *dataBuffer);
typedef void			(*_upkkXORBuffer)(byte *dataBuffer, DWORD dataLength, byte *keyBuffer, DWORD keyLength);
typedef void			(*_log)(const char *fmt, ...);

_upkkAdler32			upkkAdler32;
_upkkAllocBuffer		upkkAllocBuffer;
_upkkCloseStorage		upkkCloseStorage;
_upkkCompressBuffer		upkkCompressBuffer;
_upkkCountFiles			upkkCountFiles;
_upkkCRC32				upkkCRC32;
_upkkCreateDirectory	upkkCreateDirectory;
_upkkDecompressBuffer	upkkDecompressBuffer;
_upkkExtractData		upkkExtractData;
_upkkExtractFile		upkkExtractFile;
_upkkGetData			upkkGetData;
_upkkGetOffset			upkkGetOffset;
_upkkINIGetInt			upkkINIGetInt;
_upkkINIGetString		upkkINIGetString;
_upkkInsertData			upkkInsertData;
_upkkInsertFile			upkkInsertFile;
_upkkOpenStorage		upkkOpenStorage;
_upkkRawCloseFile		upkkRawCloseFile;
_upkkRawCreateFile		upkkRawCreateFile;
_upkkRawReadFile		upkkRawReadFile;
_upkkRawWriteFile		upkkRawWriteFile;
_upkkReadFile			upkkReadFile;
_upkkReleaseBuffer		upkkReleaseBuffer;
_upkkXORBuffer			upkkXORBuffer;
_log					log;

/// globals
archiveRow		aRow;
HMODULE			hMain;

/// API initalization.
void initUnpakkeAPI() {
	hMain = GetModuleHandle(NULL);

	upkkAdler32				= (_upkkAdler32)GetProcAddress(hMain, "upkkAdler32");
	upkkAllocBuffer			= (_upkkAllocBuffer)GetProcAddress(hMain, "upkkAllocBuffer");
	upkkCloseStorage		= (_upkkCloseStorage)GetProcAddress(hMain, "upkkCloseStorage");
	upkkCompressBuffer		= (_upkkCompressBuffer)GetProcAddress(hMain, "upkkCompressBuffer");
	upkkCountFiles			= (_upkkCountFiles)GetProcAddress(hMain, "upkkCountFiles");
	upkkCRC32				= (_upkkCRC32)GetProcAddress(hMain, "upkkCRC32");
	upkkCreateDirectory		= (_upkkCreateDirectory)GetProcAddress(hMain, "upkkCreateDirectory");
	upkkDecompressBuffer	= (_upkkDecompressBuffer)GetProcAddress(hMain, "upkkDecompressBuffer");
	upkkExtractData			= (_upkkExtractData)GetProcAddress(hMain, "upkkExtractData");
	upkkExtractFile			= (_upkkExtractFile)GetProcAddress(hMain, "upkkExtractFile");
	upkkGetData				= (_upkkGetData)GetProcAddress(hMain, "upkkGetData");
	upkkGetOffset			= (_upkkGetOffset)GetProcAddress(hMain, "upkkGetOffset");
	upkkINIGetInt			= (_upkkINIGetInt)GetProcAddress(hMain, "upkkINIGetInt");
	upkkINIGetString		= (_upkkINIGetString)GetProcAddress(hMain, "upkkINIGetString");
	upkkInsertData			= (_upkkInsertData)GetProcAddress(hMain, "upkkInsertData");
	upkkInsertFile			= (_upkkInsertFile)GetProcAddress(hMain, "upkkInsertFile");
	upkkOpenStorage			= (_upkkOpenStorage)GetProcAddress(hMain, "upkkOpenStorage");
	upkkRawCloseFile		= (_upkkRawCloseFile)GetProcAddress(hMain, "upkkRawCloseFile");
	upkkRawCreateFile		= (_upkkRawCreateFile)GetProcAddress(hMain, "upkkRawCreateFile");
	upkkRawReadFile			= (_upkkRawReadFile)GetProcAddress(hMain, "upkkRawReadFile");
	upkkRawWriteFile		= (_upkkRawWriteFile)GetProcAddress(hMain, "upkkRawWriteFile");
	upkkReadFile			= (_upkkReadFile)GetProcAddress(hMain, "upkkReadFile");
	upkkReleaseBuffer		= (_upkkReleaseBuffer)GetProcAddress(hMain, "upkkReleaseBuffer");
	upkkXORBuffer			= (_upkkXORBuffer)GetProcAddress(hMain, "upkkXORBuffer");
	log						= (_log)GetProcAddress(hMain, "log");

	if (
		upkkAdler32				== NULL||
		upkkAllocBuffer			== NULL||
		upkkCloseStorage		== NULL||
		upkkCompressBuffer		== NULL||
		upkkCountFiles			== NULL||
		upkkCRC32				== NULL||
		upkkCreateDirectory		== NULL||
		upkkDecompressBuffer	== NULL||
		upkkExtractData			== NULL||
		upkkExtractFile			== NULL||
		upkkGetData				== NULL||
		upkkGetOffset			== NULL||
		upkkINIGetInt			== NULL||
		upkkINIGetString		== NULL||
		upkkInsertData			== NULL||
		upkkInsertFile			== NULL||
		upkkOpenStorage			== NULL||
		upkkRawCloseFile		== NULL||
		upkkRawCreateFile		== NULL||
		upkkRawReadFile			== NULL||
		upkkRawWriteFile		== NULL||
		upkkReadFile			== NULL||
		upkkReleaseBuffer		== NULL||
		upkkXORBuffer			== NULL||
		log						== NULL
	) {
		MessageBox(NULL, "Cannot init the needed APIs", "ERROR!", 0);
	}
}

#endif // __MAIN_H__
